mod config;
mod modem;
#[allow(dead_code)]
mod command;

use anyhow;
use esp_idf_hal::prelude::*;
use esp_idf_hal::peripherals::Peripherals;
use esp_idf_hal::serial;

fn main() -> anyhow::Result<()> {
    esp_idf_sys::link_patches();

    let dp = Peripherals::take().unwrap();

    // LilyGo TTGO T-Call sim800l board serial pins.
    let serial_rx = dp.pins.gpio26;
    let serial_tx = dp.pins.gpio27;

    let serial_pins = serial::Pins {
        tx: serial_tx,
        rx: serial_rx,
        cts: None,
        rts: None,
    };

    // Create the serial and panic with a message ... if we can't create the serial port, then we
    // can't communicate with the sim800l module, hence we don't run anymore.
    let serial: serial::Serial<serial::UART1, _, _> = serial::Serial::new(
        dp.uart1,
        serial_pins,
        serial::config::Config::default().baudrate(Hertz(115200)),
    ).expect("Failed to create serial ... :(");

    let (tx, rx) = serial.split();
    let mut mdm = modem::Modem::new(tx, rx);

    // Unwrap all of these, without them we can't do anything anyways.
    let modem_pwrkey = dp.pins.gpio4.into_output().unwrap();
    let modem_rst = dp.pins.gpio5.into_output().unwrap();
    let modem_power = dp.pins.gpio23.into_output().unwrap();

    mdm.init(modem_pwrkey, modem_rst, modem_power)?;

    let _ = mdm.connect_to_gprs_ap(
        config::A1_GPRS_AP.apn,
        config::A1_GPRS_AP.username,
        config::A1_GPRS_AP.password,
    )?;

    loop {
        if mdm.is_gprs_attached()? {
            if mdm.tcp_is_ssl_enabled()? {
                mdm.tcp_ssl_disable()?;
            }
            let _ = mdm.get_ip_addr()?;

            println!("connecting to server!");
            let _ = mdm.tcp_set_quick_mode(true);
            let _ = mdm.tcp_connect("51.158.66.64", 9988)?;
            let _ = mdm.tcp_send("aaaaa")?;
            break;
        }
        println!("!!!!!!!!!!!!!!!! GPRS NOT ATTACHED !!!!!!!!!!!!!!!!");
    }

    Ok(())
}
